@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -51,6 +51,9 @@ class Snowflake(metaclass=abc.ABCMeta):
     Almost all :ref:`Discord models <discord_api_models>` meet this
     abstract base class.
 
+    If you want to create a snowflake on your own, consider using
+    :class:`Object`.
+
     Attributes
     -----------
     id: :class:`int`
@@ -199,9 +202,6 @@ class GuildChannel:
         bucket = self._sorting_bucket
         channels = [c for c in self.guild.channels if c._sorting_bucket == bucket]
 
-        if position >= len(channels):
-            raise InvalidArgument('Channel position cannot be greater than {}'.format(len(channels) - 1))
-
         channels.sort(key=lambda c: c.position)
 
         try:
@@ -211,8 +211,9 @@ class GuildChannel:
             # not there somehow lol
             return
         else:
+            index = next((i for i, c in enumerate(channels) if c.position >= position), -1)
             # add ourselves at our designated position
-            channels.insert(position, self)
+            channels.insert(index, self)
 
         payload = []
         for index, c in enumerate(channels):
@@ -257,6 +258,28 @@ class GuildChannel:
         else:
             await self._move(position, parent_id=parent_id, lock_permissions=lock_permissions, reason=reason)
 
+        overwrites = options.get('overwrites', None)
+        if overwrites:
+            perms = []
+            for target, perm in overwrites.items():
+                if not isinstance(perm, PermissionOverwrite):
+                    raise InvalidArgument('Expected PermissionOverwrite received {0.__name__}'.format(type(perm)))
+
+                allow, deny = perm.pair()
+                payload = {
+                    'allow': allow.value,
+                    'deny': deny.value,
+                    'id': target.id
+                }
+
+                if isinstance(target, Role):
+                    payload['type'] = 'role'
+                else:
+                    payload['type'] = 'member'
+
+                perms.append(payload)
+            options['permission_overwrites'] = perms
+
         if options:
             data = await self._state.http.edit_channel(self.id, reason=reason, **options)
             self._update(self.guild, data)
@@ -347,7 +370,7 @@ class GuildChannel:
         """Returns all of the channel's overwrites.
 
         This is returned as a dictionary where the key contains the target which
-        can be either a :class:`~discord.Role` or a :class:`~discord.Member` and the key is the
+        can be either a :class:`~discord.Role` or a :class:`~discord.Member` and the value is the
         overwrite as a :class:`~discord.PermissionOverwrite`.
 
         Returns
@@ -383,6 +406,18 @@ class GuildChannel:
         """
         return self.guild.get_channel(self.category_id)
 
+    @property
+    def permissions_synced(self):
+        """:class:`bool`: Whether or not the permissions for this channel are synced with the
+        category it belongs to.
+
+        If there is no category then this is ``False``.
+
+        .. versionadded:: 1.3
+        """
+        category = self.guild.get_channel(self.category_id)
+        return bool(category and category._overwrites == self._overwrites)
+
     def permissions_for(self, member):
         """Handles permission resolution for the current :class:`~discord.Member`.
 
@@ -418,8 +453,7 @@ class GuildChannel:
         # The operation first takes into consideration the denied
         # and then the allowed.
 
-        o = self.guild.owner
-        if o is not None and member.id == o.id:
+        if self.guild.owner_id == member.id:
             return Permissions.all()
 
         default = self.guild.default_role
@@ -490,7 +524,7 @@ class GuildChannel:
 
         Deletes the channel.
 
-        You must have :attr:`~.Permissions.manage_channels` permission to use this.
+        You must have :attr:`~Permissions.manage_channels` permission to use this.
 
         Parameters
         -----------
@@ -500,11 +534,11 @@ class GuildChannel:
 
         Raises
         -------
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have proper permissions to delete the channel.
-        :exc:`~discord.NotFound`
+        ~discord.NotFound
             The channel was not found or was already deleted.
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Deleting the channel failed.
         """
         await self._state.http.delete_channel(self.id, reason=reason)
@@ -527,7 +561,7 @@ class GuildChannel:
         If the ``overwrite`` parameter is ``None``, then the permission
         overwrites are deleted.
 
-        You must have the :attr:`~.Permissions.manage_roles` permission to use this.
+        You must have the :attr:`~Permissions.manage_roles` permission to use this.
 
         Examples
         ----------
@@ -552,8 +586,9 @@ class GuildChannel:
         -----------
         target: Union[:class:`~discord.Member`, :class:`~discord.Role`]
             The member or role to overwrite permissions for.
-        overwrite: :class:`~discord.PermissionOverwrite`
-            The permissions to allow and deny to the target.
+        overwrite: Optional[:class:`~discord.PermissionOverwrite`]
+            The permissions to allow and deny to the target, or `None` to
+            delete the overwrite.
         \*\*permissions
             A keyword argument list of permissions to set for ease of use.
             Cannot be mixed with ``overwrite``.
@@ -562,13 +597,13 @@ class GuildChannel:
 
         Raises
         -------
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have permissions to edit channel specific permissions.
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Editing channel specific permissions failed.
-        :exc:`~discord.NotFound`
+        ~discord.NotFound
             The role or member being edited is not part of the guild.
-        :exc:`~discord.InvalidArgument`
+        ~discord.InvalidArgument
             The overwrite parameter invalid or the target type was not
             :class:`~discord.Role` or :class:`~discord.Member`.
         """
@@ -624,7 +659,7 @@ class GuildChannel:
         Clones this channel. This creates a channel with the same properties
         as this channel.
 
-        .. versionadded:: 1.1.0
+        .. versionadded:: 1.1
 
         Parameters
         ------------
@@ -636,9 +671,9 @@ class GuildChannel:
 
         Raises
         -------
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have the proper permissions to create this channel.
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Creating the channel failed.
         """
         raise NotImplementedError
@@ -648,7 +683,7 @@ class GuildChannel:
 
         Creates an instant invite.
 
-        You must have the :attr:`~.Permissions.create_instant_invite` permission to
+        You must have the :attr:`~Permissions.create_instant_invite` permission to
         do this.
 
         Parameters
@@ -671,7 +706,7 @@ class GuildChannel:
 
         Raises
         -------
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Invite creation failed.
 
         Returns
@@ -688,13 +723,13 @@ class GuildChannel:
 
         Returns a list of all active instant invites from this channel.
 
-        You must have :attr:`~.Permissions.manage_guild` to get this information.
+        You must have :attr:`~Permissions.manage_guild` to get this information.
 
         Raises
         -------
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have proper permissions to get the information.
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             An error occurred while fetching the information.
 
         Returns
@@ -725,8 +760,6 @@ class Messageable(metaclass=abc.ABCMeta):
     - :class:`~discord.User`
     - :class:`~discord.Member`
     - :class:`~discord.ext.commands.Context`
-
-    This ABC must also implement :class:`~discord.abc.Snowflake`.
     """
 
     __slots__ = ()
@@ -774,11 +807,11 @@ class Messageable(metaclass=abc.ABCMeta):
 
         Raises
         --------
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Sending the message failed.
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have the proper permissions to send the message.
-        :exc:`~discord.InvalidArgument`
+        ~discord.InvalidArgument
             The ``files`` list is not of the appropriate size or
             you specified both ``file`` and ``files``.
 
@@ -871,11 +904,11 @@ class Messageable(metaclass=abc.ABCMeta):
 
         Raises
         --------
-        :exc:`~discord.NotFound`
+        ~discord.NotFound
             The specified message was not found.
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have the permissions required to get a message.
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Retrieving the message failed.
 
         Returns
@@ -901,7 +934,7 @@ class Messageable(metaclass=abc.ABCMeta):
 
         Raises
         -------
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             Retrieving the pinned messages failed.
 
         Returns
@@ -918,7 +951,7 @@ class Messageable(metaclass=abc.ABCMeta):
     def history(self, *, limit=100, before=None, after=None, around=None, oldest_first=None):
         """Returns an :class:`~discord.AsyncIterator` that enables receiving the destination's message history.
 
-        You must have :attr:`~.Permissions.read_message_history` permissions to use this.
+        You must have :attr:`~Permissions.read_message_history` permissions to use this.
 
         Examples
         ---------
@@ -960,9 +993,9 @@ class Messageable(metaclass=abc.ABCMeta):
 
         Raises
         ------
-        :exc:`~discord.Forbidden`
+        ~discord.Forbidden
             You do not have permissions to get channel message history.
-        :exc:`~discord.HTTPException`
+        ~discord.HTTPException
             The request to get message history failed.
 
         Yields
@@ -1008,11 +1041,11 @@ class Connectable(metaclass=abc.ABCMeta):
 
         Raises
         -------
-        :exc:`asyncio.TimeoutError`
+        asyncio.TimeoutError
             Could not connect to the voice channel in time.
-        :exc:`~discord.ClientException`
+        ~discord.ClientException
             You are already connected to a voice channel.
-        :exc:`~discord.opus.OpusNotLoaded`
+        ~discord.opus.OpusNotLoaded
             The opus library has not been loaded.
 
         Returns
