@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -26,15 +26,19 @@ DEALINGS IN THE SOFTWARE.
 
 import datetime
 
+from .asset import Asset
 from .enums import ActivityType, try_enum
 from .colour import Colour
+from .partial_emoji import PartialEmoji
 from .utils import _get_as_snowflake
 
 __all__ = (
+    'BaseActivity',
     'Activity',
     'Streaming',
     'Game',
     'Spotify',
+    'CustomActivity',
 )
 
 """If curious, this is the current schema for an activity.
@@ -82,10 +86,39 @@ t.ActivityFlags = {
 }
 """
 
-class _ActivityTag:
-    __slots__ = ()
+class BaseActivity:
+    """The base activity that all user-settable activities inherit from.
+    A user-settable activity is one that can be used in :meth:`Client.change_presence`.
 
-class Activity(_ActivityTag):
+    The following types currently count as user-settable:
+
+    - :class:`Activity`
+    - :class:`Game`
+    - :class:`Streaming`
+    - :class:`CustomActivity`
+
+    Note that although these types are considered user-settable by the library,
+    Discord typically ignores certain combinations of activity depending on
+    what is currently set. This behaviour may change in the future so there are
+    no guarantees on whether Discord will actually let you set these types.
+
+    .. versionadded:: 1.3
+    """
+    __slots__ = ('_created_at',)
+
+    def __init__(self, **kwargs):
+        self._created_at = kwargs.pop('created_at', None)
+
+    @property
+    def created_at(self):
+        """Optional[:class:`datetime.datetime`]: When the user started doing this activity in UTC.
+
+        .. versionadded:: 1.3
+        """
+        if self._created_at is not None:
+            return datetime.datetime.utcfromtimestamp(self._created_at / 1000)
+
+class Activity(BaseActivity):
     """Represents an activity in Discord.
 
     This could be an activity such as streaming, playing, listening
@@ -133,12 +166,16 @@ class Activity(_ActivityTag):
 
         - ``id``: A string representing the party ID.
         - ``size``: A list of up to two integer elements denoting (current_size, maximum_size).
+    emoji: Optional[:class:`PartialEmoji`]
+        The emoji that belongs to this activity.
     """
 
-    __slots__ = ('state', 'details', 'timestamps', 'assets', 'party',
-                 'flags', 'sync_id', 'session_id', 'type', 'name', 'url', 'application_id')
+    __slots__ = ('state', 'details', '_created_at', 'timestamps', 'assets', 'party',
+                 'flags', 'sync_id', 'session_id', 'type', 'name', 'url',
+                 'application_id', 'emoji')
 
     def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         self.state = kwargs.pop('state', None)
         self.details = kwargs.pop('details', None)
         self.timestamps = kwargs.pop('timestamps', {})
@@ -151,6 +188,11 @@ class Activity(_ActivityTag):
         self.sync_id = kwargs.pop('sync_id', None)
         self.session_id = kwargs.pop('session_id', None)
         self.type = try_enum(ActivityType, kwargs.pop('type', -1))
+        emoji = kwargs.pop('emoji', None)
+        if emoji is not None:
+            self.emoji = PartialEmoji.from_dict(emoji)
+        else:
+            self.emoji = None
 
     def __repr__(self):
         attrs = (
@@ -160,6 +202,7 @@ class Activity(_ActivityTag):
             'details',
             'application_id',
             'session_id',
+            'emoji',
         )
         mapped = ' '.join('%s=%r' % (attr, getattr(self, attr)) for attr in attrs)
         return '<Activity %s>' % mapped
@@ -176,6 +219,8 @@ class Activity(_ActivityTag):
 
             ret[attr] = value
         ret['type'] = int(self.type)
+        if self.emoji:
+            ret['emoji'] = self.emoji.to_dict()
         return ret
 
     @property
@@ -205,7 +250,7 @@ class Activity(_ActivityTag):
         except KeyError:
             return None
         else:
-            return 'https://cdn.discordapp.com/app-assets/{0}/{1}.png'.format(self.application_id, large_image)
+            return Asset.BASE + '/app-assets/{0}/{1}.png'.format(self.application_id, large_image)
 
     @property
     def small_image_url(self):
@@ -218,7 +263,7 @@ class Activity(_ActivityTag):
         except KeyError:
             return None
         else:
-            return 'https://cdn.discordapp.com/app-assets/{0}/{1}.png'.format(self.application_id, small_image)
+            return Asset.BASE + '/app-assets/{0}/{1}.png'.format(self.application_id, small_image)
     @property
     def large_image_text(self):
         """Optional[:class:`str`]: Returns the large image asset hover text of this activity if applicable."""
@@ -230,7 +275,7 @@ class Activity(_ActivityTag):
         return self.assets.get('small_text', None)
 
 
-class Game(_ActivityTag):
+class Game(BaseActivity):
     """A slimmed down version of :class:`Activity` that represents a Discord game.
 
     This is typically displayed via **Playing** on the official Discord client.
@@ -271,6 +316,7 @@ class Game(_ActivityTag):
     __slots__ = ('name', '_end', '_start')
 
     def __init__(self, name, **extra):
+        super().__init__(**extra)
         self.name = name
 
         try:
@@ -341,7 +387,7 @@ class Game(_ActivityTag):
     def __hash__(self):
         return hash(self.name)
 
-class Streaming(_ActivityTag):
+class Streaming(BaseActivity):
     """A slimmed down version of :class:`Activity` that represents a Discord streaming status.
 
     This is typically displayed via **Streaming** on the official Discord client.
@@ -366,23 +412,35 @@ class Streaming(_ActivityTag):
 
     Attributes
     -----------
-    name: :class:`str`
+    platform: :class:`str`
+        Where the user is streaming from (ie. YouTube, Twitch).
+
+        .. versionadded:: 1.3
+
+    name: Optional[:class:`str`]
         The stream's name.
-    url: :class:`str`
-        The stream's URL. Currently only twitch.tv URLs are supported. Anything else is silently
-        discarded.
     details: Optional[:class:`str`]
-        If provided, typically the game the streamer is playing.
+        Same as :attr:`name`
+    game: Optional[:class:`str`]
+        The game being streamed.
+
+        .. versionadded:: 1.3
+
+    url: :class:`str`
+        The stream's URL.
     assets: :class:`dict`
         A dictionary comprising of similar keys than those in :attr:`Activity.assets`.
     """
 
-    __slots__ = ('name', 'url', 'details', 'assets')
+    __slots__ = ('platform', 'name', 'game', 'url', 'details', 'assets')
 
     def __init__(self, *, name, url, **extra):
-        self.name = name
+        super().__init__(**extra)
+        self.platform = name
+        self.name = extra.pop('details', name)
+        self.game = extra.pop('state', None)
         self.url = url
-        self.details = extra.pop('details', None)
+        self.details = extra.pop('details', self.name) # compatibility
         self.assets = extra.pop('assets', {})
 
     @property
@@ -457,7 +515,8 @@ class Spotify:
             Returns the string 'Spotify'.
     """
 
-    __slots__ = ('_state', '_details', '_timestamps', '_assets', '_party', '_sync_id', '_session_id')
+    __slots__ = ('_state', '_details', '_timestamps', '_assets', '_party', '_sync_id', '_session_id',
+                 '_created_at')
 
     def __init__(self, **data):
         self._state = data.pop('state', None)
@@ -467,6 +526,7 @@ class Spotify:
         self._party = data.pop('party', {})
         self._sync_id = data.pop('sync_id')
         self._session_id = data.pop('session_id')
+        self._created_at = data.pop('created_at', None)
 
     @property
     def type(self):
@@ -476,6 +536,15 @@ class Spotify:
         """
         return ActivityType.listening
 
+    @property
+    def created_at(self):
+        """Optional[:class:`datetime.datetime`]: When the user started listening in UTC.
+
+        .. versionadded:: 1.3
+        """
+        if self._created_at is not None:
+            return datetime.datetime.utcfromtimestamp(self._created_at / 1000)
+
     @property
     def colour(self):
         """Returns the Spotify integration colour, as a :class:`Colour`.
@@ -582,6 +651,96 @@ class Spotify:
         """:class:`str`: The party ID of the listening party."""
         return self._party.get('id', '')
 
+class CustomActivity(BaseActivity):
+    """Represents a Custom activity from Discord.
+
+    .. container:: operations
+
+        .. describe:: x == y
+
+            Checks if two activities are equal.
+
+        .. describe:: x != y
+
+            Checks if two activities are not equal.
+
+        .. describe:: hash(x)
+
+            Returns the activity's hash.
+
+        .. describe:: str(x)
+
+            Returns the custom status text.
+
+    .. versionadded:: 1.3
+
+    Attributes
+    -----------
+    name: Optional[:class:`str`]
+        The custom activity's name.
+    emoji: Optional[`:class:`PartialEmoji`]
+        The emoji to pass to the activity, if any.
+    """
+
+    __slots__ = ('name', 'emoji', 'state')
+
+    def __init__(self, name, *, emoji=None, **extra):
+        self.name = name
+        self.state = extra.pop('state', None)
+        if self.name == 'Custom Status':
+            self.name = self.state
+
+        if emoji is None:
+            self.emoji = emoji
+        else:
+            self.emoji = PartialEmoji.from_dict(emoji)
+
+    @property
+    def type(self):
+        """Returns the activity's type. This is for compatibility with :class:`Activity`.
+
+        It always returns :attr:`ActivityType.custom`.
+        """
+        return ActivityType.custom
+
+    def to_dict(self):
+        if self.name == self.state:
+            o = {
+                'type': ActivityType.custom.value,
+                'state': self.name,
+                'name': 'Custom Status',
+            }
+        else:
+            o = {
+                'type': ActivityType.custom.value,
+                'name': self.name,
+            }
+
+        if self.emoji:
+            o['emoji'] = self.emoji.to_dict()
+        return o
+
+    def __eq__(self, other):
+        return (isinstance(other, CustomActivity) and other.name == self.name and other.emoji == self.emoji)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __hash__(self):
+        return hash((self.name, str(self.emoji)))
+
+    def __str__(self):
+        if self.emoji:
+            if self.name:
+                return '%s %s' % (self.emoji, self.name)
+            return str(self.emoji)
+        else:
+            return str(self.name)
+
+    def __repr__(self):
+        return '<CustomActivity name={0.name!r} emoji={0.emoji!r}>'.format(self)
+
+
 def create_activity(data):
     if not data:
         return None
@@ -591,6 +750,13 @@ def create_activity(data):
         if 'application_id' in data or 'session_id' in data:
             return Activity(**data)
         return Game(**data)
+    elif game_type is ActivityType.custom:
+        try:
+            name = data.pop('name')
+        except KeyError:
+            return Activity(**data)
+        else:
+            return CustomActivity(name=name, **data)
     elif game_type is ActivityType.streaming:
         if 'url' in data:
             return Streaming(**data)
