@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -33,7 +33,7 @@ import datetime
 import discord
 
 from .errors import *
-from .cooldowns import Cooldown, BucketType, CooldownMapping
+from .cooldowns import Cooldown, BucketType, CooldownMapping, MaxConcurrency
 from . import converter as converters
 from ._types import _BaseCommand
 from .cog import Cog
@@ -48,14 +48,18 @@ __all__ = (
     'has_permissions',
     'has_any_role',
     'check',
+    'check_any',
     'bot_has_role',
     'bot_has_permissions',
     'bot_has_any_role',
     'cooldown',
+    'max_concurrency',
     'dm_only',
     'guild_only',
     'is_owner',
     'is_nsfw',
+    'has_guild_permissions',
+    'bot_has_guild_permissions'
 )
 
 def wrap_callback(coro):
@@ -87,6 +91,9 @@ def hooked_wrapped_callback(command, ctx, coro):
             ctx.command_failed = True
             raise CommandInvokeError(exc) from exc
         finally:
+            if command._max_concurrency is not None:
+                await command._max_concurrency.release(ctx)
+
             await command.call_after_hooks(ctx)
         return ret
     return wrapped
@@ -102,22 +109,22 @@ def _convert_to_bool(argument):
 
 class _CaseInsensitiveDict(dict):
     def __contains__(self, k):
-        return super().__contains__(k.lower())
+        return super().__contains__(k.casefold())
 
     def __delitem__(self, k):
-        return super().__delitem__(k.lower())
+        return super().__delitem__(k.casefold())
 
     def __getitem__(self, k):
-        return super().__getitem__(k.lower())
+        return super().__getitem__(k.casefold())
 
     def get(self, k, default=None):
-        return super().get(k.lower(), default)
+        return super().get(k.casefold(), default)
 
     def pop(self, k, default=None):
-        return super().pop(k.lower(), default)
+        return super().pop(k.casefold(), default)
 
     def __setitem__(self, k, v):
-        super().__setitem__(k.lower(), v)
+        super().__setitem__(k.casefold(), v)
 
 class Command(_BaseCommand):
     r"""A class that implements the protocol for a bot text command.
@@ -148,6 +155,8 @@ class Command(_BaseCommand):
     parent: Optional[:class:`Command`]
         The parent command that this command belongs to. ``None`` if there
         isn't one.
+    cog: Optional[:class:`Cog`]
+        The cog that this command belongs to. ``None`` if there isn't one.
     checks: List[Callable[..., :class:`bool`]]
         A list of predicates that verifies if the command could be executed
         with the given :class:`.Context` as the sole parameter. If an exception
@@ -243,6 +252,13 @@ class Command(_BaseCommand):
         finally:
             self._buckets = CooldownMapping(cooldown)
 
+        try:
+            max_concurrency = func.__commands_max_concurrency__
+        except AttributeError:
+            max_concurrency = kwargs.get('max_concurrency')
+        finally:
+            self._max_concurrency = max_concurrency
+
         self.ignore_extra = kwargs.get('ignore_extra', True)
         self.cooldown_after_parsing = kwargs.get('cooldown_after_parsing', False)
         self.cog = None
@@ -276,6 +292,40 @@ class Command(_BaseCommand):
             if value.annotation is converters.Greedy:
                 raise TypeError('Unparameterized Greedy[...] is disallowed in signature.')
 
+    def add_check(self, func):
+        """Adds a check to the command.
+
+        This is the non-decorator interface to :func:`.check`.
+
+        .. versionadded:: 1.3
+
+        Parameters
+        -----------
+        func
+            The function that will be used as a check.
+        """
+
+        self.checks.append(func)
+
+    def remove_check(self, func):
+        """Removes a check from the command.
+
+        This function is idempotent and will not raise an exception
+        if the function is not in the command's checks.
+
+        .. versionadded:: 1.3
+
+        Parameters
+        -----------
+        func
+            The function to remove from the checks.
+        """
+
+        try:
+            self.checks.remove(func)
+        except ValueError:
+            pass
+
     def update(self, **kwargs):
         """Updates :class:`Command` instance with updated attribute.
 
@@ -285,6 +335,24 @@ class Command(_BaseCommand):
         """
         self.__init__(self.callback, **dict(self.__original_kwargs__, **kwargs))
 
+    async def __call__(self, *args, **kwargs):
+        """|coro|
+
+        Calls the internal callback that the command holds.
+
+        .. note::
+
+            This bypasses all mechanisms -- including checks, converters,
+            invoke hooks, cooldowns, etc. You must take care to pass
+            the proper arguments and types to this function.
+
+        .. versionadded:: 1.3
+        """
+        if self.cog is not None:
+            return await self.callback(self.cog, *args, **kwargs)
+        else:
+            return await self.callback(*args, **kwargs)
+
     def _ensure_assignment_on_copy(self, other):
         other._before_invoke = self._before_invoke
         other._after_invoke = self._after_invoke
@@ -292,6 +360,9 @@ class Command(_BaseCommand):
             other.checks = self.checks.copy()
         if self._buckets.valid and not other._buckets.valid:
             other._buckets = self._buckets.copy()
+        if self._max_concurrency != other._max_concurrency:
+            other._max_concurrency = self._max_concurrency.copy()
+
         try:
             other.on_error = self.on_error
         except AttributeError:
@@ -345,7 +416,7 @@ class Command(_BaseCommand):
             pass
         else:
             if module is not None and (module.startswith('discord.') and not module.endswith('converter')):
-                converter = getattr(converters, converter.__name__ + 'Converter')
+                converter = getattr(converters, converter.__name__ + 'Converter', converter)
 
         try:
             if inspect.isclass(converter):
@@ -462,10 +533,10 @@ class Command(_BaseCommand):
             previous = view.index
 
             view.skip_ws()
-            argument = view.get_quoted_word()
             try:
+                argument = view.get_quoted_word()
                 value = await self.do_conversion(ctx, converter, argument, param)
-            except CommandError:
+            except (CommandError, ArgumentParsingError):
                 view.index = previous
                 break
             else:
@@ -478,10 +549,10 @@ class Command(_BaseCommand):
     async def _transform_greedy_var_pos(self, ctx, param, converter):
         view = ctx.view
         previous = view.index
-        argument = view.get_quoted_word()
         try:
+            argument = view.get_quoted_word()
             value = await self.do_conversion(ctx, converter, argument, param)
-        except CommandError:
+        except (CommandError, ArgumentParsingError):
             view.index = previous
             raise RuntimeError() from None # break loop
         else:
@@ -529,7 +600,7 @@ class Command(_BaseCommand):
 
         For example in commands ``?a b c test``, the parents are ``[c, b, a]``.
 
-        .. versionadded:: 1.1.0
+        .. versionadded:: 1.1
         """
         entries = []
         command = self
@@ -619,13 +690,6 @@ class Command(_BaseCommand):
             if not view.eof:
                 raise TooManyArguments('Too many arguments passed to ' + self.qualified_name)
 
-    async def _verify_checks(self, ctx):
-        if not self.enabled:
-            raise DisabledCommand('{0.name} command is disabled'.format(self))
-
-        if not await self.can_run(ctx):
-            raise CheckFailure('The check functions for command {0.qualified_name} failed.'.format(self))
-
     async def call_before_hooks(self, ctx):
         # now that we're done preparing we can call the pre-command hooks
         # first, call the command local hook:
@@ -675,7 +739,9 @@ class Command(_BaseCommand):
 
     async def prepare(self, ctx):
         ctx.command = self
-        await self._verify_checks(ctx)
+
+        if not await self.can_run(ctx):
+            raise CheckFailure('The check functions for command {0.qualified_name} failed.'.format(self))
 
         if self.cooldown_after_parsing:
             await self._parse_arguments(ctx)
@@ -684,6 +750,9 @@ class Command(_BaseCommand):
             self._prepare_cooldowns(ctx)
             await self._parse_arguments(ctx)
 
+        if self._max_concurrency is not None:
+            await self._max_concurrency.acquire(ctx)
+
         await self.call_before_hooks(ctx)
 
     def is_on_cooldown(self, ctx):
@@ -893,7 +962,11 @@ class Command(_BaseCommand):
         """|coro|
 
         Checks if the command can be executed by checking all the predicates
-        inside the :attr:`.checks` attribute.
+        inside the :attr:`.checks` attribute. This also checks whether the
+        command is disabled.
+
+        .. versionchanged:: 1.3
+            Checks whether the command is disabled or not
 
         Parameters
         -----------
@@ -912,6 +985,9 @@ class Command(_BaseCommand):
             A boolean indicating if the command can be invoked.
         """
 
+        if not self.enabled:
+            raise DisabledCommand('{0.name} command is disabled'.format(self))
+
         original = ctx.command
         ctx.command = self
 
@@ -1248,7 +1324,7 @@ def group(name=None, **attrs):
     This is similar to the :func:`.command` decorator but the ``cls``
     parameter is set to :class:`Group` by default.
 
-    .. versionchanged:: 1.1.0
+    .. versionchanged:: 1.1
         The ``cls`` parameter can now be passed.
     """
 
@@ -1269,9 +1345,27 @@ def check(predicate):
     will be propagated while those subclassed will be sent to
     :func:`.on_command_error`.
 
+    A special attribute named ``predicate`` is bound to the value
+    returned by this decorator to retrieve the predicate passed to the
+    decorator. This allows the following introspection and chaining to be done:
+
+    .. code-block:: python3
+
+        def owner_or_permissions(**perms):
+            original = commands.has_permissions(**perms).predicate
+            async def extended_check(ctx):
+                if ctx.guild is None:
+                    return False
+                return ctx.guild.owner_id == ctx.author.id or await original(ctx)
+            return commands.check(extended_check)
+
     .. note::
 
-        These functions can either be regular functions or coroutines.
+        The function returned by ``predicate`` is **always** a coroutine,
+        even if the original function was not a coroutine.
+
+    .. versionchanged:: 1.3
+        The ``predicate`` attribute was added.
 
     Examples
     ---------
@@ -1318,8 +1412,85 @@ def check(predicate):
             func.__commands_checks__.append(predicate)
 
         return func
+
+    if inspect.iscoroutinefunction(predicate):
+        decorator.predicate = predicate
+    else:
+        @functools.wraps(predicate)
+        async def wrapper(ctx):
+            return predicate(ctx)
+        decorator.predicate = wrapper
+
     return decorator
 
+def check_any(*checks):
+    """A :func:`check` that is added that checks if any of the checks passed
+    will pass, i.e. using logical OR.
+
+    If all checks fail then :exc:`.CheckAnyFailure` is raised to signal the failure.
+    It inherits from :exc:`.CheckFailure`.
+
+    .. note::
+
+        The ``predicate`` attribute for this function **is** a coroutine.
+
+    .. versionadded:: 1.3
+
+    Parameters
+    ------------
+    \*checks: Callable[[:class:`Context`], :class:`bool`]
+        An argument list of checks that have been decorated with
+        the :func:`check` decorator.
+
+    Raises
+    -------
+    TypeError
+        A check passed has not been decorated with the :func:`check`
+        decorator.
+
+    Examples
+    ---------
+
+    Creating a basic check to see if it's the bot owner or
+    the server owner:
+
+    .. code-block:: python3
+
+        def is_guild_owner():
+            def predicate(ctx):
+                return ctx.guild is not None and ctx.guild.owner_id == ctx.author.id
+            return commands.check(predicate)
+
+        @bot.command()
+        @commands.check_any(commands.is_owner(), is_guild_owner())
+        async def only_for_owners(ctx):
+            await ctx.send('Hello mister owner!')
+    """
+
+    unwrapped = []
+    for wrapped in checks:
+        try:
+            pred = wrapped.predicate
+        except AttributeError:
+            raise TypeError('%r must be wrapped by commands.check decorator' % wrapped) from None
+        else:
+            unwrapped.append(pred)
+
+    async def predicate(ctx):
+        errors = []
+        for func in unwrapped:
+            try:
+                value = await func(ctx)
+            except CheckFailure as e:
+                errors.append(e)
+            else:
+                if value:
+                    return True
+        # if we're here, all checks failed
+        raise CheckAnyFailure(unwrapped, errors)
+
+    return check(predicate)
+
 def has_role(item):
     """A :func:`.check` that is added that checks if the member invoking the
     command has the role specified via the name or ID specified.
@@ -1336,7 +1507,7 @@ def has_role(item):
     is missing a role, or :exc:`.NoPrivateMessage` if it is used in a private message.
     Both inherit from :exc:`.CheckFailure`.
 
-    .. versionchanged:: 1.1.0
+    .. versionchanged:: 1.1
 
         Raise :exc:`.MissingRole` or :exc:`.NoPrivateMessage`
         instead of generic :exc:`.CheckFailure`
@@ -1372,7 +1543,7 @@ def has_any_role(*items):
     is missing all roles, or :exc:`.NoPrivateMessage` if it is used in a private message.
     Both inherit from :exc:`.CheckFailure`.
 
-    .. versionchanged:: 1.1.0
+    .. versionchanged:: 1.1
 
         Raise :exc:`.MissingAnyRole` or :exc:`.NoPrivateMessage`
         instead of generic :exc:`.CheckFailure`
@@ -1411,7 +1582,7 @@ def bot_has_role(item):
     is missing the role, or :exc:`.NoPrivateMessage` if it is used in a private message.
     Both inherit from :exc:`.CheckFailure`.
 
-    .. versionchanged:: 1.1.0
+    .. versionchanged:: 1.1
 
         Raise :exc:`.BotMissingRole` or :exc:`.NoPrivateMessage`
         instead of generic :exc:`.CheckFailure`
@@ -1440,7 +1611,7 @@ def bot_has_any_role(*items):
     is missing all roles, or :exc:`.NoPrivateMessage` if it is used in a private message.
     Both inherit from :exc:`.CheckFailure`.
 
-    .. versionchanged:: 1.1.0
+    .. versionchanged:: 1.1
 
         Raise :exc:`.BotMissingAnyRole` or :exc:`.NoPrivateMessage`
         instead of generic checkfailure
@@ -1461,6 +1632,9 @@ def has_permissions(**perms):
     """A :func:`.check` that is added that checks if the member has all of
     the permissions necessary.
 
+    Note that this check operates on the current channel permissions, not the
+    guild wide permissions.
+
     The permissions passed in must be exactly like the properties shown under
     :class:`.discord.Permissions`.
 
@@ -1517,6 +1691,49 @@ def bot_has_permissions(**perms):
 
     return check(predicate)
 
+def has_guild_permissions(**perms):
+    """Similar to :func:`.has_permissions`, but operates on guild wide
+    permissions instead of the current channel permissions.
+
+    If this check is called in a DM context, it will raise an
+    exception, :exc:`.NoPrivateMessage`.
+
+    .. versionadded:: 1.3
+    """
+    def predicate(ctx):
+        if not ctx.guild:
+            raise NoPrivateMessage
+
+        permissions = ctx.author.guild_permissions
+        missing = [perm for perm, value in perms.items() if getattr(permissions, perm, None) != value]
+
+        if not missing:
+            return True
+
+        raise MissingPermissions(missing)
+
+    return check(predicate)
+
+def bot_has_guild_permissions(**perms):
+    """Similar to :func:`.has_guild_permissions`, but checks the bot
+    members guild permissions.
+
+    .. versionadded:: 1.3
+    """
+    def predicate(ctx):
+        if not ctx.guild:
+            raise NoPrivateMessage
+
+        permissions = ctx.me.guild_permissions
+        missing = [perm for perm, value in perms.items() if getattr(permissions, perm, None) != value]
+
+        if not missing:
+            return True
+
+        raise BotMissingPermissions(missing)
+
+    return check(predicate)
+
 def dm_only():
     """A :func:`.check` that indicates this command must only be used in a
     DM context. Only private messages are allowed when
@@ -1525,7 +1742,7 @@ def dm_only():
     This check raises a special exception, :exc:`.PrivateMessageOnly`
     that is inherited from :exc:`.CheckFailure`.
 
-    .. versionadded:: 1.1.0
+    .. versionadded:: 1.1
     """
 
     def predicate(ctx):
@@ -1574,7 +1791,7 @@ def is_nsfw():
     This check raises a special exception, :exc:`.NSFWChannelRequired`
     that is derived from :exc:`.CheckFailure`.
 
-    .. versionchanged:: 1.1.0
+    .. versionchanged:: 1.1
 
         Raise :exc:`.NSFWChannelRequired` instead of generic :exc:`.CheckFailure`.
         DM channels will also now pass this check.
@@ -1592,16 +1809,9 @@ def cooldown(rate, per, type=BucketType.default):
 
     A cooldown allows a command to only be used a specific amount
     of times in a specific time frame. These cooldowns can be based
-    either on a per-guild, per-channel, per-user, or global basis.
+    either on a per-guild, per-channel, per-user, per-role or global basis.
     Denoted by the third argument of ``type`` which must be of enum
-    type ``BucketType`` which could be either:
-
-    - ``BucketType.default`` for a global basis.
-    - ``BucketType.user`` for a per-user basis.
-    - ``BucketType.guild`` for a per-guild basis.
-    - ``BucketType.channel`` for a per-channel basis.
-    - ``BucketType.member`` for a per-member basis.
-    - ``BucketType.category`` for a per-category basis.
+    type :class:`.BucketType`.
 
     If a cooldown is triggered, then :exc:`.CommandOnCooldown` is triggered in
     :func:`.on_command_error` and the local error handler.
@@ -1614,7 +1824,7 @@ def cooldown(rate, per, type=BucketType.default):
         The number of times a command can be used before triggering a cooldown.
     per: :class:`float`
         The amount of seconds to wait for a cooldown when it's been triggered.
-    type: ``BucketType``
+    type: :class:`.BucketType`
         The type of cooldown to have.
     """
 
@@ -1625,3 +1835,36 @@ def cooldown(rate, per, type=BucketType.default):
             func.__commands_cooldown__ = Cooldown(rate, per, type)
         return func
     return decorator
+
+def max_concurrency(number, per=BucketType.default, *, wait=False):
+    """A decorator that adds a maximum concurrency to a :class:`.Command` or its subclasses.
+
+    This enables you to only allow a certain number of command invocations at the same time,
+    for example if a command takes too long or if only one user can use it at a time. This
+    differs from a cooldown in that there is no set waiting period or token bucket -- only
+    a set number of people can run the command.
+
+    .. versionadded:: 1.3
+
+    Parameters
+    -------------
+    number: :class:`int`
+        The maximum number of invocations of this command that can be running at the same time.
+    per: :class:`.BucketType`
+        The bucket that this concurrency is based on, e.g. ``BucketType.guild`` would allow
+        it to be used up to ``number`` times per guild.
+    wait: :class:`bool`
+        Whether the command should wait for the queue to be over. If this is set to ``False``
+        then instead of waiting until the command can run again, the command raises
+        :exc:`.MaxConcurrencyReached` to its error handler. If this is set to ``True``
+        then the command waits until it can be executed.
+    """
+
+    def decorator(func):
+        value = MaxConcurrency(number, per=per, wait=wait)
+        if isinstance(func, Command):
+            func._max_concurrency = value
+        else:
+            func.__commands_max_concurrency__ = value
+        return func
+    return decorator
