@@ -1,4 +1,5 @@
 import asyncio
+import datetime
 import aiohttp
 import websockets
 import discord
@@ -7,8 +8,6 @@ import logging
 
 from discord.backoff import ExponentialBackoff
 
-MAX_ASYNCIO_SECONDS = 3456000
-
 log = logging.getLogger(__name__)
 
 class Loop:
@@ -45,6 +44,8 @@ class Loop:
             raise ValueError('count must be greater than 0 or None.')
 
         self.change_interval(seconds=seconds, minutes=minutes, hours=hours)
+        self._last_iteration = None
+        self._next_iteration = None
 
         if not inspect.iscoroutinefunction(self.coro):
             raise TypeError('Expected coroutine function, not {0.__name__!r}.'.format(type(self.coro)))
@@ -62,10 +63,17 @@ class Loop:
     async def _loop(self, *args, **kwargs):
         backoff = ExponentialBackoff()
         await self._call_loop_function('before_loop')
+        sleep_until = discord.utils.sleep_until
+        self._next_iteration = datetime.datetime.now(datetime.timezone.utc)
         try:
             while True:
+                self._last_iteration = self._next_iteration
+                self._next_iteration = self._get_next_sleep_time()
                 try:
                     await self.coro(*args, **kwargs)
+                    now = datetime.datetime.now(datetime.timezone.utc)
+                    if now > self._next_iteration:
+                        self._next_iteration = now
                 except self._valid_exception as exc:
                     if not self.reconnect:
                         raise
@@ -77,7 +85,7 @@ class Loop:
                     if self._current_loop == self.count:
                         break
 
-                    await asyncio.sleep(self._sleep)
+                    await sleep_until(self._next_iteration)
         except asyncio.CancelledError:
             self._is_being_cancelled = True
             raise
@@ -103,6 +111,18 @@ class Loop:
         """:class:`int`: The current iteration of the loop."""
         return self._current_loop
 
+    @property
+    def next_iteration(self):
+        """Optional[:class:`datetime.datetime`]: When the next iteration of the loop will occur.
+
+        .. versionadded:: 1.3
+        """
+        if self._task is None and self._sleep:
+            return None
+        elif self._task and self._task.done() or self._stop_next_iteration:
+            return None
+        return self._next_iteration
+
     def start(self, *args, **kwargs):
         r"""Starts the internal task in the event loop.
 
@@ -149,7 +169,7 @@ class Loop:
             before stopping via :meth:`clear_exception_types` or
             use :meth:`cancel` instead.
 
-        .. versionadded:: 1.2.0
+        .. versionadded:: 1.2
         """
         if self._task and not self._task.done():
             self._stop_next_iteration = True
@@ -186,8 +206,8 @@ class Loop:
             self._task.add_done_callback(restart_when_over)
             self._task.cancel()
 
-    def add_exception_type(self, exc):
-        r"""Adds an exception type to be handled during the reconnect logic.
+    def add_exception_type(self, *exceptions):
+        r"""Adds exception types to be handled during the reconnect logic.
 
         By default the exception types handled are those handled by
         :meth:`discord.Client.connect`\, which includes a lot of internet disconnection
@@ -198,21 +218,22 @@ class Loop:
 
         Parameters
         ------------
-        exc: Type[:class:`BaseException`]
-            The exception class to handle.
+        \*exceptions: Type[:class:`BaseException`]
+            An argument list of exception classes to handle.
 
         Raises
         --------
         TypeError
-            The exception passed is either not a class or not inherited from :class:`BaseException`.
+            An exception passed is either not a class or not inherited from :class:`BaseException`.
         """
 
-        if not inspect.isclass(exc):
-            raise TypeError('{0!r} must be a class.'.format(exc))
-        if not issubclass(exc, BaseException):
-            raise TypeError('{0!r} must inherit from BaseException.'.format(exc))
+        for exc in exceptions:
+            if not inspect.isclass(exc):
+                raise TypeError('{0!r} must be a class.'.format(exc))
+            if not issubclass(exc, BaseException):
+                raise TypeError('{0!r} must inherit from BaseException.'.format(exc))
 
-        self._valid_exception = (*self._valid_exception, exc)
+        self._valid_exception = (*self._valid_exception, *exceptions)
 
     def clear_exception_types(self):
         """Removes all exception types that are handled.
@@ -223,22 +244,22 @@ class Loop:
         """
         self._valid_exception = tuple()
 
-    def remove_exception_type(self, exc):
-        """Removes an exception type from being handled during the reconnect logic.
+    def remove_exception_type(self, *exceptions):
+        r"""Removes exception types from being handled during the reconnect logic.
 
         Parameters
         ------------
-        exc: Type[:class:`BaseException`]
-            The exception class to handle.
+        \*exceptions: Type[:class:`BaseException`]
+            An argument list of exception classes to handle.
 
         Returns
         ---------
         :class:`bool`
-            Whether it was successfully removed.
+            Whether all exceptions were successfully removed.
         """
         old_length = len(self._valid_exception)
-        self._valid_exception = tuple(x for x in self._valid_exception if x is not exc)
-        return len(self._valid_exception) != old_length
+        self._valid_exception = tuple(x for x in self._valid_exception if x not in exceptions)
+        return len(self._valid_exception) == old_length - len(exceptions)
 
     def get_task(self):
         """Optional[:class:`asyncio.Task`]: Fetches the internal task or ``None`` if there isn't one running."""
@@ -251,7 +272,7 @@ class Loop:
     def failed(self):
         """:class:`bool`: Whether the internal task has failed.
 
-        .. versionadded:: 1.2.0
+        .. versionadded:: 1.2
         """
         return self._has_failed
 
@@ -308,6 +329,9 @@ class Loop:
         self._after_loop = coro
         return coro
 
+    def _get_next_sleep_time(self):
+        return self._last_iteration + datetime.timedelta(seconds=self._sleep)
+
     def change_interval(self, *, seconds=0, minutes=0, hours=0):
         """Changes the interval for the sleep time.
 
@@ -316,7 +340,7 @@ class Loop:
             This only applies on the next loop iteration. If it is desirable for the change of interval
             to be applied right away, cancel the task with :meth:`cancel`.
 
-        .. versionadded:: 1.2.0
+        .. versionadded:: 1.2
 
         Parameters
         ------------
@@ -334,10 +358,6 @@ class Loop:
         """
 
         sleep = seconds + (minutes * 60.0) + (hours * 3600.0)
-        if sleep >= MAX_ASYNCIO_SECONDS:
-            fmt = 'Total number of seconds exceeds asyncio imposed limit of {0} seconds.'
-            raise ValueError(fmt.format(MAX_ASYNCIO_SECONDS))
-
         if sleep < 0:
             raise ValueError('Total number of seconds cannot be less than zero.')
 
@@ -348,7 +368,7 @@ class Loop:
 
 def loop(*, seconds=0, minutes=0, hours=0, count=None, reconnect=True, loop=None):
     """A decorator that schedules a task in the background for you with
-    optional reconnect logic.
+    optional reconnect logic. The decorator returns a :class:`Loop`.
 
     Parameters
     ------------
@@ -375,11 +395,6 @@ def loop(*, seconds=0, minutes=0, hours=0, count=None, reconnect=True, loop=None
         An invalid value was given.
     TypeError
         The function was not a coroutine.
-
-    Returns
-    ---------
-    :class:`Loop`
-        The loop helper that handles the background task.
     """
     def decorator(func):
         return Loop(func, seconds=seconds, minutes=minutes, hours=hours,
