@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -31,12 +31,17 @@ import io
 
 from . import utils
 from .reaction import Reaction
-from .emoji import Emoji, PartialEmoji
+from .emoji import Emoji
+from .partial_emoji import PartialEmoji
 from .calls import CallMessage
 from .enums import MessageType, try_enum
 from .errors import InvalidArgument, ClientException, HTTPException
 from .embeds import Embed
 from .member import Member
+from .flags import MessageFlags
+from .file import File
+from .utils import escape_mentions
+
 
 class Attachment:
     """Represents an attachment from Discord.
@@ -48,9 +53,9 @@ class Attachment:
     size: :class:`int`
         The attachment size in bytes.
     height: Optional[:class:`int`]
-        The attachment's height, in pixels. Only applicable to images.
+        The attachment's height, in pixels. Only applicable to images and videos.
     width: Optional[:class:`int`]
-        The attachment's width, in pixels. Only applicable to images.
+        The attachment's width, in pixels. Only applicable to images and videos.
     filename: :class:`str`
         The attachment's filename.
     url: :class:`str`
@@ -130,7 +135,7 @@ class Attachment:
 
         Retrieves the content of this attachment as a :class:`bytes` object.
 
-        .. versionadded:: 1.1.0
+        .. versionadded:: 1.1
 
         Parameters
         -----------
@@ -160,6 +165,32 @@ class Attachment:
         data = await self._http.get_from_cdn(url)
         return data
 
+    async def to_file(self):
+        """|coro|
+
+        Converts the attachment into a :class:`File` suitable for sending via
+        :meth:`abc.Messageable.send`.
+
+        .. versionadded:: 1.3
+
+        Raises
+        ------
+        HTTPException
+            Downloading the attachment failed.
+        Forbidden
+            You do not have permissions to access this attachment
+        NotFound
+            The attachment was deleted.
+
+        Returns
+        -------
+        :class:`File`
+            The attachment as a file suitable for sending.
+        """
+
+        data = await self.read()
+        return File(io.BytesIO(data), filename=self.filename)
+
 def flatten_handlers(cls):
     prefix = len('_handle_')
     cls._HANDLERS = {
@@ -236,6 +267,11 @@ class Message:
         A list of attachments given to a message.
     pinned: :class:`bool`
         Specifies if the message is currently pinned.
+    flags: :class:`MessageFlags`
+        Extra features of the message.
+
+        .. versionadded:: 1.3
+
     reactions : List[:class:`Reaction`]
         Reactions to a message. Reactions can be either custom emoji or standard unicode emoji.
     activity: Optional[:class:`dict`]
@@ -262,7 +298,7 @@ class Message:
                  'mention_everyone', 'embeds', 'id', 'mentions', 'author',
                  '_cs_channel_mentions', '_cs_raw_mentions', 'attachments',
                  '_cs_clean_content', '_cs_raw_channel_mentions', 'nonce', 'pinned',
-                 'role_mentions', '_cs_raw_role_mentions', 'type', 'call',
+                 'role_mentions', '_cs_raw_role_mentions', 'type', 'call', 'flags',
                  '_cs_system_content', '_cs_guild', '_state', 'reactions',
                  'application', 'activity')
 
@@ -279,19 +315,20 @@ class Message:
         self._edited_timestamp = utils.parse_time(data['edited_timestamp'])
         self.type = try_enum(MessageType, data['type'])
         self.pinned = data['pinned']
+        self.flags = MessageFlags._from_value(data.get('flags', 0))
         self.mention_everyone = data['mention_everyone']
         self.tts = data['tts']
         self.content = data['content']
         self.nonce = data.get('nonce')
 
-        for handler in ('author', 'member', 'mentions', 'mention_roles', 'call'):
+        for handler in ('author', 'member', 'mentions', 'mention_roles', 'call', 'flags'):
             try:
                 getattr(self, '_handle_%s' % handler)(data[handler])
             except KeyError:
                 continue
 
     def __repr__(self):
-        return '<Message id={0.id} channel={0.channel!r} type={0.type!r} author={0.author!r}>'.format(self)
+        return '<Message id={0.id} channel={0.channel!r} type={0.type!r} author={0.author!r} flags={0.flags!r}>'.format(self)
 
     def _try_patch(self, data, key, transform=None):
         try:
@@ -337,6 +374,18 @@ class Message:
 
         return reaction
 
+    def _clear_emoji(self, emoji):
+        to_check = str(emoji)
+        for index, reaction in enumerate(self.reactions):
+            if str(reaction.emoji) == to_check:
+                break
+        else:
+            # didn't find anything so just return
+            return
+
+        del self.reactions[index]
+        return reaction
+
     def _update(self, data):
         handlers = self._HANDLERS
         for key, value in data.items():
@@ -360,6 +409,9 @@ class Message:
     def _handle_pinned(self, value):
         self.pinned = value
 
+    def _handle_flags(self, value):
+        self.flags = MessageFlags._from_value(value)
+
     def _handle_application(self, value):
         self.application = value
 
@@ -403,22 +455,29 @@ class Message:
         # ourselves to a more "partial" member object.
         author = self.author
         try:
+            # Update member reference
             if author.joined_at is None:
                 author.joined_at = utils.parse_time(member.get('joined_at'))
         except AttributeError:
+            # It's a user here
+            # TODO: consider adding to cache here
             self.author = Member._from_message(message=self, data=member)
 
     def _handle_mentions(self, mentions):
-        self.mentions = []
-        if self.guild is None:
-            self.mentions = [self._state.store_user(m) for m in mentions]
+        self.mentions = r = []
+        guild = self.guild
+        state = self._state
+        if guild is None:
+            self.mentions = [state.store_user(m) for m in mentions]
             return
 
         for mention in filter(None, mentions):
             id_search = int(mention['id'])
-            member = self.guild.get_member(id_search)
+            member = guild.get_member(id_search)
             if member is not None:
-                self.mentions.append(member)
+                r.append(member)
+            else:
+                r.append(Member._try_upgrade(data=mention, guild=guild, state=state))
 
     def _handle_mention_roles(self, role_mentions):
         self.role_mentions = []
@@ -532,17 +591,7 @@ class Message:
 
         pattern = re.compile('|'.join(transformations.keys()))
         result = pattern.sub(repl, self.content)
-
-        transformations = {
-            '@everyone': '@\u200beveryone',
-            '@here': '@\u200bhere'
-        }
-
-        def repl2(obj):
-            return transformations.get(obj.group(0), '')
-
-        pattern = re.compile('|'.join(transformations.keys()))
-        return pattern.sub(repl2, result)
+        return escape_mentions(result)
 
     @property
     def created_at(self):
@@ -560,6 +609,13 @@ class Message:
         guild_id = getattr(self.guild, 'id', '@me')
         return 'https://discordapp.com/channels/{0}/{1.channel.id}/{1.id}'.format(guild_id, self)
 
+    def is_system(self):
+        """:class:`bool`: Whether the message is a system message.
+
+        .. versionadded:: 1.3
+        """
+        return self.type is not MessageType.default
+
     @utils.cached_slot_property('_cs_system_content')
     def system_content(self):
         r"""A property that returns the content that is rendered
@@ -662,6 +718,9 @@ class Message:
         if self.type is MessageType.premium_guild_tier_3:
             return '{0.author.name} just boosted the server! {0.guild} has achieved **Level 3!**'.format(self)
 
+        if self.type is MessageType.channel_follow_add:
+            return '{0.author.name} has added {0.content} to this channel'.format(self)
+
     async def delete(self, *, delay=None):
         """|coro|
 
@@ -671,25 +730,27 @@ class Message:
         delete other people's messages, you need the :attr:`~Permissions.manage_messages`
         permission.
 
-        .. versionchanged:: 1.1.0
+        .. versionchanged:: 1.1
             Added the new ``delay`` keyword-only parameter.
 
         Parameters
         -----------
         delay: Optional[:class:`float`]
             If provided, the number of seconds to wait in the background
-            before deleting the message.
+            before deleting the message. If the deletion fails then it is silently ignored.
 
         Raises
         ------
         Forbidden
             You do not have proper permissions to delete the message.
+        NotFound
+            The message was deleted already
         HTTPException
             Deleting the message failed.
         """
         if delay is not None:
             async def delete():
-                await asyncio.sleep(delay, loop=self._state.loop)
+                await asyncio.sleep(delay)
                 try:
                     await self._state.http.delete_message(self.channel.id, self.id)
                 except HTTPException:
@@ -706,6 +767,9 @@ class Message:
 
         The content must be able to be transformed into a string via ``str(content)``.
 
+        .. versionchanged:: 1.3
+            The ``suppress`` keyword-only parameter was added.
+
         Parameters
         -----------
         content: Optional[:class:`str`]
@@ -714,6 +778,11 @@ class Message:
         embed: Optional[:class:`Embed`]
             The new embed to replace the original with.
             Could be ``None`` to remove the embed.
+        suppress: :class:`bool`
+            Whether to suppress embeds for the message. This removes
+            all the embeds if set to ``True``. If set to ``False``
+            this brings the embeds back if they were suppressed.
+            Using this parameter requires :attr:`~.Permissions.manage_messages`.
         delete_after: Optional[:class:`float`]
             If provided, the number of seconds to wait in the background
             before deleting the message we just edited. If the deletion fails,
@@ -723,6 +792,9 @@ class Message:
         -------
         HTTPException
             Editing the message failed.
+        Forbidden
+            Tried to suppress a message without permissions or
+            edited a message's content or embed that isn't yours.
         """
 
         try:
@@ -741,16 +813,44 @@ class Message:
             if embed is not None:
                 fields['embed'] = embed.to_dict()
 
-        data = await self._state.http.edit_message(self.channel.id, self.id, **fields)
-        self._update(data)
-
         try:
-            delete_after = fields['delete_after']
+            suppress = fields.pop('suppress')
         except KeyError:
             pass
         else:
-            if delete_after is not None:
-                await self.delete(delay=delete_after)
+             flags = MessageFlags._from_value(self.flags.value)
+             flags.suppress_embeds = suppress
+             fields['flags'] = flags.value
+
+        delete_after = fields.pop('delete_after', None)
+
+        if fields:
+            data = await self._state.http.edit_message(self.channel.id, self.id, **fields)
+            self._update(data)
+
+        if delete_after is not None:
+            await self.delete(delay=delete_after)
+
+    async def publish(self):
+        """|coro|
+
+        Publishes this message to your announcement channel.
+
+        You must have the :attr:`~Permissions.manage_messages` permission to use this.
+
+        .. note::
+
+            This can only be used by non-bot accounts.
+
+        Raises
+        -------
+        Forbidden
+            You do not have the proper permissions to publish this message.
+        HTTPException
+            Publishing the message failed.
+        """
+
+        await self._state.http.publish_message(self.channel.id, self.id)
 
     async def pin(self):
         """|coro|
@@ -865,6 +965,37 @@ class Message:
         else:
             await self._state.http.remove_reaction(self.channel.id, self.id, emoji, member.id)
 
+    async def clear_reaction(self, emoji):
+        """|coro|
+
+        Clears a specific reaction from the message.
+
+        The emoji may be a unicode emoji or a custom guild :class:`Emoji`.
+
+        You need the :attr:`~Permissions.manage_messages` permission to use this.
+
+        .. versionadded:: 1.3
+
+        Parameters
+        -----------
+        emoji: Union[:class:`Emoji`, :class:`Reaction`, :class:`PartialEmoji`, :class:`str`]
+            The emoji to clear.
+
+        Raises
+        --------
+        HTTPException
+            Clearing the reaction failed.
+        Forbidden
+            You do not have the proper permissions to clear the reaction.
+        NotFound
+            The emoji you specified was not found.
+        InvalidArgument
+            The emoji parameter is invalid.
+        """
+
+        emoji = self._emoji_reaction(emoji)
+        await self._state.http.clear_single_reaction(self.channel.id, self.id, emoji)
+
     @staticmethod
     def _emoji_reaction(emoji):
         if isinstance(emoji, Reaction):
