@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -33,11 +33,13 @@ import logging
 import math
 import weakref
 import inspect
+import gc
 
 from .guild import Guild
-from .activity import _ActivityTag
+from .activity import BaseActivity
 from .user import User, ClientUser
-from .emoji import Emoji, PartialEmoji
+from .emoji import Emoji
+from .partial_emoji import PartialEmoji
 from .message import Message
 from .relationship import Relationship
 from .channel import *
@@ -48,9 +50,11 @@ from .enums import ChannelType, try_enum, Status, Enum
 from . import utils
 from .embeds import Embed
 from .object import Object
+from .invite import Invite
 
 class ListenerType(Enum):
     chunk = 0
+    query_members = 1
 
 Listener = namedtuple('Listener', ('type', 'future', 'predicate'))
 log = logging.getLogger(__name__)
@@ -60,7 +64,10 @@ class ConnectionState:
     def __init__(self, *, dispatch, chunker, handlers, syncer, http, loop, **options):
         self.loop = loop
         self.http = http
-        self.max_messages = max(options.get('max_messages', 5000), 100)
+        self.max_messages = options.get('max_messages', 1000)
+        if self.max_messages is not None and self.max_messages <= 0:
+            self.max_messages = 1000
+
         self.dispatch = dispatch
         self.chunker = chunker
         self.syncer = syncer
@@ -70,12 +77,15 @@ class ConnectionState:
         self._ready_task = None
         self._fetch_offline = options.get('fetch_offline_members', True)
         self.heartbeat_timeout = options.get('heartbeat_timeout', 60.0)
+        self.guild_subscriptions = options.get('guild_subscriptions', True)
+        # Only disable cache if both fetch_offline and guild_subscriptions are off.
+        self._cache_members = (self._fetch_offline or self.guild_subscriptions)
         self._listeners = []
 
         activity = options.get('activity', None)
         if activity:
-            if not isinstance(activity, _ActivityTag):
-                raise TypeError('activity parameter must be one of Game, Streaming, or Activity.')
+            if not isinstance(activity, BaseActivity):
+                raise TypeError('activity parameter must derive from BaseActivity.')
 
             activity = activity.to_dict()
 
@@ -108,7 +118,12 @@ class ConnectionState:
         self._private_channels = OrderedDict()
         # extra dict to look up private channels by user id
         self._private_channels_by_user = {}
-        self._messages = deque(maxlen=self.max_messages)
+        self._messages = self.max_messages and deque(maxlen=self.max_messages)
+
+        # In cases of large deallocations the GC should be called explicitly
+        # To free the memory more immediately, especially true when it comes
+        # to reconnect loops which cause mass allocations and deallocations.
+        gc.collect()
 
     def process_listeners(self, listener_type, argument, result):
         removed = []
@@ -203,6 +218,10 @@ class ConnectionState:
 
         del guild
 
+        # Much like clear(), if we have a massive deallocation
+        # then it's better to explicitly call the GC
+        gc.collect()
+
     @property
     def emojis(self):
         return list(self._emojis.values())
@@ -249,7 +268,7 @@ class ConnectionState:
             self._private_channels_by_user.pop(channel.recipient.id, None)
 
     def _get_message(self, msg_id):
-        return utils.find(lambda m: m.id == msg_id, reversed(self._messages))
+        return utils.find(lambda m: m.id == msg_id, reversed(self._messages)) if self._messages else None
 
     def _add_guild_from_data(self, guild):
         guild = Guild(data=guild, state=self)
@@ -286,21 +305,51 @@ class ConnectionState:
         # wait for the chunks
         if chunks:
             try:
-                await utils.sane_wait_for(chunks, timeout=len(chunks) * 30.0, loop=self.loop)
+                await utils.sane_wait_for(chunks, timeout=len(chunks) * 30.0)
             except asyncio.TimeoutError:
                 log.info('Somehow timed out waiting for chunks.')
 
+    async def query_members(self, guild, query, limit, cache):
+        guild_id = guild.id
+        ws = self._get_websocket(guild_id)
+        if ws is None:
+            raise RuntimeError('Somehow do not have a websocket for this guild_id')
+
+        # Limits over 1000 cannot be supported since
+        # the main use case for this is guild_subscriptions being disabled
+        # and they don't receive GUILD_MEMBER events which make computing
+        # member_count impossible. The only way to fix it is by limiting
+        # the limit parameter to 1 to 1000.
+        future = self.receive_member_query(guild_id, query)
+        try:
+            # start the query operation
+            await ws.request_chunks(guild_id, query, limit)
+            members = await asyncio.wait_for(future, timeout=5.0)
+
+            if cache:
+                for member in members:
+                    guild._add_member(member)
+
+            return members
+        except asyncio.TimeoutError:
+            log.info('Timed out waiting for chunks with query %r and limit %d for guild_id %d', query, limit, guild_id)
+            raise
+
     async def _delay_ready(self):
         try:
             launch = self._ready_state.launch
 
             # only real bots wait for GUILD_CREATE streaming
             if self.is_bot:
-                while not launch.is_set():
+                while True:
                     # this snippet of code is basically waiting 2 seconds
                     # until the last GUILD_CREATE was sent
-                    launch.set()
-                    await asyncio.sleep(2, loop=self.loop)
+                    try:
+                        await asyncio.wait_for(launch.wait(), timeout=2.0)
+                    except asyncio.TimeoutError:
+                        break
+                    else:
+                        launch.clear()
 
             guilds = next(zip(*self._ready_state.guilds), [])
             if self._fetch_offline:
@@ -368,7 +417,8 @@ class ConnectionState:
         channel, _ = self._get_guild_channel(data)
         message = Message(channel=channel, data=data, state=self)
         self.dispatch('message', message)
-        self._messages.append(message)
+        if self._messages is not None:
+            self._messages.append(message)
         if channel and channel.__class__ is TextChannel:
             channel.last_message_id = message.id
 
@@ -377,13 +427,16 @@ class ConnectionState:
         found = self._get_message(raw.message_id)
         raw.cached_message = found
         self.dispatch('raw_message_delete', raw)
-        if found is not None:
+        if self._messages is not None and found is not None:
             self.dispatch('message_delete', found)
             self._messages.remove(found)
 
     def parse_message_delete_bulk(self, data):
         raw = RawBulkMessageDeleteEvent(data)
-        found_messages = [message for message in self._messages if message.id in raw.message_ids]
+        if self._messages:
+            found_messages = [message for message in self._messages if message.id in raw.message_ids]
+        else:
+            found_messages = []
         raw.cached_messages = found_messages
         self.dispatch('raw_bulk_message_delete', raw)
         if found_messages:
@@ -407,7 +460,14 @@ class ConnectionState:
         emoji = data['emoji']
         emoji_id = utils._get_as_snowflake(emoji, 'id')
         emoji = PartialEmoji.with_state(self, animated=emoji.get('animated', False), id=emoji_id, name=emoji['name'])
-        raw = RawReactionActionEvent(data, emoji)
+        raw = RawReactionActionEvent(data, emoji, 'REACTION_ADD')
+
+        member_data = data.get('member')
+        if member_data:
+            guild = self._get_guild(raw.guild_id)
+            raw.member = Member(data=member_data, guild=guild, state=self)
+        else:
+            raw.member = None
         self.dispatch('raw_reaction_add', raw)
 
         # rich interface here
@@ -415,7 +475,8 @@ class ConnectionState:
         if message is not None:
             emoji = self._upgrade_partial_emoji(emoji)
             reaction = message._add_reaction(data, emoji, raw.user_id)
-            user = self._get_reaction_user(message.channel, raw.user_id)
+            user = raw.member or self._get_reaction_user(message.channel, raw.user_id)
+
             if user:
                 self.dispatch('reaction_add', reaction, user)
 
@@ -433,7 +494,7 @@ class ConnectionState:
         emoji = data['emoji']
         emoji_id = utils._get_as_snowflake(emoji, 'id')
         emoji = PartialEmoji.with_state(self, animated=emoji.get('animated', False), id=emoji_id, name=emoji['name'])
-        raw = RawReactionActionEvent(data, emoji)
+        raw = RawReactionActionEvent(data, emoji, 'REACTION_REMOVE')
         self.dispatch('raw_reaction_remove', raw)
 
         message = self._get_message(raw.message_id)
@@ -448,6 +509,23 @@ class ConnectionState:
                 if user:
                     self.dispatch('reaction_remove', reaction, user)
 
+    def parse_message_reaction_remove_emoji(self, data):
+        emoji = data['emoji']
+        emoji_id = utils._get_as_snowflake(emoji, 'id')
+        emoji = PartialEmoji.with_state(self, animated=emoji.get('animated', False), id=emoji_id, name=emoji['name'])
+        raw = RawReactionClearEmojiEvent(data, emoji)
+        self.dispatch('raw_reaction_clear_emoji', raw)
+
+        message = self._get_message(raw.message_id)
+        if message is not None:
+            try:
+                reaction = message._clear_emoji(emoji)
+            except (AttributeError, ValueError): # eventual consistency lol
+                pass
+            else:
+                if reaction:
+                    self.dispatch('reaction_clear_emoji', reaction)
+
     def parse_presence_update(self, data):
         guild_id = utils._get_as_snowflake(data, 'guild_id')
         guild = self._get_guild(guild_id)
@@ -477,6 +555,14 @@ class ConnectionState:
     def parse_user_update(self, data):
         self.user._update(data)
 
+    def parse_invite_create(self, data):
+        invite = Invite.from_gateway(state=self, data=data)
+        self.dispatch('invite_create', invite)
+
+    def parse_invite_delete(self, data):
+        invite = Invite.from_gateway(state=self, data=data)
+        self.dispatch('invite_delete', invite)
+
     def parse_channel_delete(self, data):
         guild = self._get_guild(utils._get_as_snowflake(data, 'guild_id'))
         channel_id = int(data['id'])
@@ -583,7 +669,8 @@ class ConnectionState:
             return
 
         member = Member(guild=guild, data=data, state=self)
-        guild._add_member(member)
+        if self._cache_members:
+            guild._add_member(member)
         guild._member_count += 1
         self.dispatch('member_join', member)
 
@@ -645,7 +732,7 @@ class ConnectionState:
         await self.chunker(guild)
         if chunks:
             try:
-                await utils.sane_wait_for(chunks, timeout=len(chunks), loop=self.loop)
+                await utils.sane_wait_for(chunks, timeout=len(chunks))
             except asyncio.TimeoutError:
                 log.info('Somehow timed out waiting for chunks.')
 
@@ -672,7 +759,7 @@ class ConnectionState:
                 # so we say.
                 try:
                     state = self._ready_state
-                    state.launch.clear()
+                    state.launch.set()
                     state.guilds.append((guild, unavailable))
                 except AttributeError:
                     # the _ready_state attribute is only there during
@@ -720,7 +807,8 @@ class ConnectionState:
             return
 
         # do a cleanup of the messages cache
-        self._messages = deque((msg for msg in self._messages if msg.guild != guild), maxlen=self.max_messages)
+        if self._messages is not None:
+            self._messages = deque((msg for msg in self._messages if msg.guild != guild), maxlen=self.max_messages)
 
         self._remove_guild(guild)
         self.dispatch('guild_remove', guild)
@@ -788,15 +876,17 @@ class ConnectionState:
     def parse_guild_members_chunk(self, data):
         guild_id = int(data['guild_id'])
         guild = self._get_guild(guild_id)
-        members = data.get('members', [])
-        for member in members:
-            m = Member(guild=guild, data=member, state=self)
-            existing = guild.get_member(m.id)
-            if existing is None or existing.joined_at is None:
-                guild._add_member(m)
-
+        members = [Member(guild=guild, data=member, state=self) for member in data.get('members', [])]
         log.info('Processed a chunk for %s members in guild ID %s.', len(members), guild_id)
+        if self._cache_members:
+            for member in members:
+                existing = guild.get_member(member.id)
+                if existing is None or existing.joined_at is None:
+                    guild._add_member(member)
+
         self.process_listeners(ListenerType.chunk, guild, len(members))
+        names = [x.name.lower() for x in members]
+        self.process_listeners(ListenerType.query_members, (guild_id, names), members)
 
     def parse_guild_integrations_update(self, data):
         guild = self._get_guild(int(data['guild_id']))
@@ -926,6 +1016,15 @@ class ConnectionState:
         self._listeners.append(listener)
         return future
 
+    def receive_member_query(self, guild_id, query):
+        def predicate(args, *, guild_id=guild_id, query=query.lower()):
+            request_guild_id, names = args
+            return request_guild_id == guild_id and all(n.startswith(query) for n in names)
+        future = self.loop.create_future()
+        listener = Listener(ListenerType.query_members, future, predicate)
+        self._listeners.append(listener)
+        return future
+
 class AutoShardedConnectionState(ConnectionState):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -945,37 +1044,35 @@ class AutoShardedConnectionState(ConnectionState):
         # wait for the chunks
         if chunks:
             try:
-                await utils.sane_wait_for(chunks, timeout=len(chunks) * 30.0, loop=self.loop)
+                await utils.sane_wait_for(chunks, timeout=len(chunks) * 30.0)
             except asyncio.TimeoutError:
                 log.info('Somehow timed out waiting for chunks.')
 
     async def _delay_ready(self):
         launch = self._ready_state.launch
-        while not launch.is_set():
+        while True:
             # this snippet of code is basically waiting 2 seconds
             # until the last GUILD_CREATE was sent
-            launch.set()
-            await asyncio.sleep(2.0 * self.shard_count, loop=self.loop)
+            try:
+                await asyncio.wait_for(launch.wait(), timeout=2.0)
+            except asyncio.TimeoutError:
+                break
+            else:
+                launch.clear()
 
-        if self._fetch_offline:
-            guilds = sorted(self._ready_state.guilds, key=lambda g: g[0].shard_id)
+        guilds = sorted(self._ready_state.guilds, key=lambda g: g[0].shard_id)
 
-            for shard_id, sub_guilds_info in itertools.groupby(guilds, key=lambda g: g[0].shard_id):
-                sub_guilds, sub_available = zip(*sub_guilds_info)
+        for shard_id, sub_guilds_info in itertools.groupby(guilds, key=lambda g: g[0].shard_id):
+            sub_guilds, sub_available = zip(*sub_guilds_info)
+            if self._fetch_offline:
                 await self.request_offline_members(sub_guilds, shard_id=shard_id)
 
-                for guild, unavailable in zip(sub_guilds, sub_available):
-                    if unavailable is False:
-                        self.dispatch('guild_available', guild)
-                    else:
-                        self.dispatch('guild_join', guild)
-                self.dispatch('shard_ready', shard_id)
-        else:
-            for guild, unavailable in self._ready_state.guilds:
+            for guild, unavailable in zip(sub_guilds, sub_available):
                 if unavailable is False:
                     self.dispatch('guild_available', guild)
                 else:
                     self.dispatch('guild_join', guild)
+            self.dispatch('shard_ready', shard_id)
 
         # remove the state
         try:
